﻿//
//   (c) Copyright Agilent Technologies 2011
//              All rights reserved.
//
//  Customer shall have the personal, non-
// transferable rights to use, copy or modify
// this SAMPLE PROGRAM for Customer's internal
// operations. Customer shall use the SAMPLE
// PROGRAM solely and exclusively for its own
// purpose and shall not license, lease, market
// or distribute the SAMPLE PROGRAM or modification
// or any part thereof.
//
//  Agilent shall not be liable for the quality,
// performance or behavior of the SAMPLE PROGRAM.
// Agilent especially disclaims that the operation
// of the SAMPLE PROGRAM shall be uninterrupted or
// error free. This SAMPLE PROGRAM is provided
// AS IS.
//
//  AGILENT DISCLAIMS THE IMPLIED WARRANTIES OF
// MERCHANTABILITY AND FITNESS FOR A PARTICULAR
// PURPOSE.
//
//  Agilent shall not be liable for any infringement
// of any patent, trademark, copyright or other
// proprietary rights by the SAMPLE PROGRAM or
// its use. Agilent does not warrant that the SAMPLE
// PROGRAM is free from infringements or such
// rights of third parties. However, Agilent will not
// knowingly infringe or deliver a software that
// infringes the patent, trademark, copyright or
// other proprietary right of a third party.
//
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Ivi.Visa.Interop;

namespace B2900CsSample
{
    class SmuControl
    {
        private ResourceManager ioMgr = new ResourceManager();
        private FormattedIO488 instrument = null;
        private string visaResStr = "";
        private int maxChannel = 0;
        private string modelName = "";
        private string errorMsg = "";
        private const int WAIT_BEFORE_QUERY = 10;

        //Public Sub New()
        //End Sub

        public const int COLUMN_COUNT = 6;

        public int MaxSmuChannel
        {
            get {return maxChannel;}
        }

        public string ModelName
        {
            get {return modelName;}
        }

        public string ErrorMessage
        {
            get {return errorMsg;}
        }

        public bool OpenDevice(string address)
        {
            try
            {
                if (instrument != null) {
                    CloseDevice();
                }

                string buf;
                
                // Open
                visaResStr = address;
                instrument = new FormattedIO488();
                instrument.IO = (IMessage)ioMgr.Open(address);
                instrument.IO.TerminationCharacterEnabled = true;
                System.Threading.Thread.Sleep(200);

                // Check Model Name
                instrument.WriteString("*IDN?");
                System.Threading.Thread.Sleep(WAIT_BEFORE_QUERY);
                buf = instrument.ReadString().Trim();
                string[] tokens = buf.Split(',');
                if (tokens.Length < 2)
                {
                    maxChannel = 0;
                    modelName = "";
                    errorMsg = "Illegal *IDN response.";
                    CloseDevice();
                    return false;
                }

                modelName = buf.Split(',')[1];
                switch (modelName)
                {
                    case "B2901A":
                    case "B2911A":
                        maxChannel = 1;
                        break;
                    case "B2902A":
                    case "B2912A":
                        maxChannel = 2;
                        break;
                    default:
                        maxChannel = 0;
                        errorMsg = "Unknown model is found.";
                        CloseDevice();
                        return false;
                }
 
                // Check Language Mode
                instrument.WriteString(":SYST:LANG?");
                System.Threading.Thread.Sleep(WAIT_BEFORE_QUERY);
                buf = instrument.ReadString().Trim();
                if (buf != "\"DEF\"")
                {
                    maxChannel = 0;
                    errorMsg = "Not support 2400 language mode.";
                    CloseDevice();
                    return false;
                }

                // Reset
                instrument.WriteString("*RST");
                instrument.WriteString("*CLS");
            }
            catch ( Exception ex )
            {
                instrument = null;
                errorMsg = ex.Message;
                return false;
            }
            return true;
        }

        public void CloseDevice()
        {
            if (instrument != null) instrument.IO.Close();
            instrument = null;
        }

        private bool ErrorCheck()
        {
            try
            {
                string ret;
                instrument.WriteString(":SYST:ERR:COUNT?");
                System.Threading.Thread.Sleep(WAIT_BEFORE_QUERY);
                ret = instrument.ReadString().Trim();
                if ( int.Parse(ret) > 0 )
                {
                    instrument.WriteString(":SYST:ERR:ALL?");
                    System.Threading.Thread.Sleep(WAIT_BEFORE_QUERY);
                    errorMsg = instrument.ReadString();
                    return false;
                }
            }
            catch ( Exception ex )
            {
                errorMsg = ex.Message;
                return false;
            }
            return true;
        }

        public bool SetSweepChannel( int ch, double startv, double stopv, int nofstep, double compi )
        {
            try
            {
                instrument.WriteString(":SENS" + ch.ToString() + ":CURR:PROT " + compi.ToString());
                instrument.WriteString(":SOUR" + ch.ToString() + ":VOLT " + startv.ToString());
                instrument.WriteString(":SOUR" + ch.ToString() + ":FUNC:MODE VOLT");
                instrument.WriteString(":SOUR" + ch.ToString() + ":FUNC:SHAP DC");
                instrument.WriteString(":SOUR" + ch.ToString() + ":VOLT:MODE SWE");
                instrument.WriteString(":SOUR" + ch.ToString() + ":SWE:DIR UP");
                instrument.WriteString(":SOUR" + ch.ToString() + ":SWE:STA SING");
                instrument.WriteString(":SOUR" + ch.ToString() + ":SWE:RANG AUTO");
                instrument.WriteString(":SOUR" + ch.ToString() + ":SWE:SPAC LIN");
                instrument.WriteString(":SOUR" + ch.ToString() + ":VOLT:STAR " + startv.ToString());
                instrument.WriteString(":SOUR" + ch.ToString() + ":VOLT:STOP " + stopv.ToString());
                instrument.WriteString(":SOUR" + ch.ToString() + ":SWE:POIN " + nofstep.ToString());
                instrument.WriteString(":OUTP" + ch.ToString() + ":STAT ON");
                instrument.WriteString(":ARM" + ch.ToString() + ":COUN 1");
                instrument.WriteString(":ARM" + ch.ToString() + ":DEL 0.0");
                instrument.WriteString(":ARM" + ch.ToString() + ":SOUR AINT");
                instrument.WriteString(":TRIG" + ch.ToString() + ":COUN " + nofstep.ToString());
                instrument.WriteString(":TRIG" + ch.ToString() + ":DEL 0.0");
                instrument.WriteString(":TRIG" + ch.ToString() + ":SOUR AINT");
                return ErrorCheck();
            }
            catch ( Exception ex )
            {
                errorMsg = ex.Message;
                return false;
            }
        }

        public bool SetConstChannel( int ch, double forcev, double compi )
        {
            try
            {
                instrument.WriteString(":SENS" + ch.ToString() + ":CURR:PROT " + compi.ToString());
                instrument.WriteString(":SOUR" + ch.ToString() + ":VOLT " + forcev.ToString());
                instrument.WriteString(":SOUR" + ch.ToString() + ":FUNC:MODE VOLT");
                instrument.WriteString(":SOUR" + ch.ToString() + ":FUNC:SHAP DC");
                instrument.WriteString(":SOUR" + ch.ToString() + ":VOLT:MODE FIX");
                instrument.WriteString(":OUTP" + ch.ToString() + ":STAT ON");
                return ErrorCheck();
            }
            catch ( Exception ex )
            {
                errorMsg = ex.Message;
                return false;
            }
        }

        public bool Initiate( int[] ch )
        {
            try
            {
                instrument.WriteString(":FORM ASC");
                instrument.WriteString(":FORM:ELEM:SENS VOLT,CURR,RES,TIME,STAT,SOUR");
                string chlist = "(@";
                for ( int i = 0 ; i < ch.Length ; i++ ) 
                {
                    if ( i > 0 ) chlist += ",";
                    chlist += ch[i].ToString();
                }
                chlist += ")";
                instrument.WriteString(":INIT " + chlist);
                return ErrorCheck();
            }
            catch ( Exception ex )
            {
                errorMsg = ex.Message;
                return false;
            }
        }

        public bool GetData( int ch, bool fromStart, int len, ref double[,] data, out int actrow )
        {
            actrow = 0;
            try
            {
                string ret;
                bool isBusy;
                do
                {
                    instrument.WriteString(":STAT:OPER:COND?");
                    System.Threading.Thread.Sleep(WAIT_BEFORE_QUERY);
                    ret = instrument.ReadString().Trim();
                    isBusy = ((int.Parse(ret) & 0x0010) == 0);

                    instrument.WriteString(":SENS" + ch.ToString() + ":DATA? " + (fromStart ? "STAR," : "CURR,") + len.ToString());
                    System.Threading.Thread.Sleep(WAIT_BEFORE_QUERY);
                    ret = instrument.ReadString().Trim();
                    string[] ary = ret.Split(',');
                    int row, col;
                    for (row = 0; row < ary.Length / COLUMN_COUNT; row++)
                    {
                        if (double.Parse(ary[row * COLUMN_COUNT]) >= 9.91e37) break;
                        for (col = 0; col < COLUMN_COUNT; col++)
                        {
                            data[row,col] = double.Parse(ary[row * COLUMN_COUNT + col]);
                        }
                    }
                    actrow = row;

                } while (actrow == 0 && isBusy);

                return ErrorCheck();
            }
            catch ( Exception ex )
            {
                errorMsg = ex.Message;
                return false;
            }
        }
    }
}
